use crate::common_usings::*;
use pkmn_lib_interface::PkmnResult;

pub struct AfterYou {}

impl AfterYou {
    pub const fn get_const_name() -> &'static str {
        "after_you"
    }
}

impl Script for AfterYou {
    fn new() -> Self {
        Self {}
    }

    fn get_name(&self) -> &'static str {
        Self::get_const_name()
    }

    fn get_capabilities(&self) -> &[ScriptCapabilities] {
        &[ScriptCapabilities::OnSecondaryEffect]
    }

    fn on_secondary_effect(&self, mv: ExecutingMove, target: Pokemon, hit: u8) -> PkmnResult<()> {
        if !target
            .battle()?
            .unwrap()
            .choice_queue()
            .move_pokemon_choice_next(&target)?
        {
            mv.get_hit_data(&target, hit)?.fail()?
        }
        Ok(())
    }

    fn as_any(&self) -> &dyn Any {
        self
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use alloc::rc::Rc;
    use pkmn_lib_interface::app_interface::{
        MockBattle, MockChoiceQueue, MockExecutingMove, MockHitData, MockPokemon,
    };

    #[test]
    fn move_pokemon_choice_next_gets_called_once() {
        let mut battle = MockBattle::new();
        battle.expect_choice_queue().once().return_once_st(move || {
            let mut choice_queue = MockChoiceQueue::new();
            choice_queue
                .expect_move_pokemon_choice_next()
                .once()
                .returning_st(move |_| Ok(true));
            Rc::new(choice_queue)
        });

        let battle = Rc::new(battle);
        let mut target = MockPokemon::new();
        target
            .expect_battle()
            .once()
            .return_once_st(move || Ok(Some(battle)));
        let target = Rc::new(target);

        let script = AfterYou::new();
        script
            .on_secondary_effect(Rc::new(MockExecutingMove::new()), target, 0)
            .unwrap();
    }

    #[test]
    fn fails_when_move_pokemon_choice_next_can_not_move() {
        let mut battle = MockBattle::new();
        battle.expect_choice_queue().once().return_once_st(move || {
            let mut choice_queue = MockChoiceQueue::new();
            choice_queue
                .expect_move_pokemon_choice_next()
                .once()
                .returning_st(move |_| Ok(false));
            Rc::new(choice_queue)
        });

        let battle = Rc::new(battle);
        let mut target = MockPokemon::new();
        target
            .expect_battle()
            .once()
            .return_once_st(move || Ok(Some(battle)));
        let target = Rc::new(target);

        let mut mv = MockExecutingMove::new();
        mv.expect_get_hit_data().once().returning_st(move |_, _| {
            let mut hit = MockHitData::new();
            hit.expect_fail().returning(|| Ok(())).once();
            Ok(Rc::new(hit))
        });
        let mv = Rc::new(mv);

        let script = AfterYou::new();
        script.on_secondary_effect(mv, target, 0).unwrap();
    }
}
