use alloc::boxed::Box;
#[cfg(not(feature = "mock_data"))]
use alloc::vec::Vec;
use core::mem;

#[repr(C)]
pub struct FFIArray<T> {
    ptr: *mut T,
    len: usize,
}

impl<T> FFIArray<T> {
    pub fn new(v: &[T]) -> Self {
        let boxed_slice: Box<&[T]> = Box::new(v);
        let r = Self {
            ptr: boxed_slice.as_ptr() as *mut T,
            len: boxed_slice.len(),
        };
        mem::forget(boxed_slice);
        r
    }

    #[cfg(not(feature = "mock_data"))]
    pub(crate) fn from_u64(v: u64) -> Self {
        let v: (u32, u32) = unsafe { core::mem::transmute(v) };
        Self {
            ptr: v.0 as *mut T,
            len: v.1 as usize,
        }
    }
}

impl<T> Default for FFIArray<T> {
    fn default() -> Self {
        Self {
            ptr: core::ptr::null_mut(),
            len: 0,
        }
    }
}

#[cfg(not(feature = "mock_data"))]
impl<T> FFIArray<T> {
    fn delete(&self) {
        unsafe {
            drop(Vec::from_raw_parts(self.ptr, self.len, self.len));
        }
    }

    pub(crate) fn ptr(&self) -> *mut T {
        self.ptr
    }
    pub(crate) fn len(&self) -> usize {
        self.len
    }
}

#[no_mangle]
#[cfg(not(feature = "mock_data"))]
extern "wasm" fn delete_ffi_array(a: FFIArray<u8>) {
    a.delete()
}
